%{

#include "bison-actions.h"

%}

// IDs de los tokens generados desde Flex:

%token ADD
%token SUB
%token MUL
%token DIV
%token POW
%token FUNCTION
%token IF
%token ELSE
%token FOR
%token WHILE
%token PRINT
%token SEMICOLON
%token COMPOSITE
%token SQUOTE
%token DERIVATIVE
%token INTEGRATE
%token BETWEEN
%token AND_INT
%token EVALUATE
%token IN
%token OPEN_BRACES
%token OPEN_BRACKETS
%token CLOSE_BRACES
%token CLOSE_BRACKETS
%token OPEN_PARENTHESIS
%token CLOSE_PARENTHESIS
%token ASSIGN
%token EQUAL
%token AND
%token OR
%token NOT
%token ADD_ASSIGN
%token INCREMENT
%token SUB_ASSIGN
%token DECREMENT
%token MUL_ASSIGN
%token DIV_ASSIGN
%token DQUOTE
%token COMMA
%token COLON
%token LESSER
%token LESSER_EQUAL
%token GREATER
%token GREATER_EQUAL
%token NOT_EQUAL
%token VAR
%token ERR
%token TAYLOR
%token ESCAPESTRING
%token DOUBLE
%token INTEGER
%token ID

// Reglas de asociatividad y precedencia (de menor a mayor):

%precedence CLOSE_PARENTHESIS
%precedence ELSE

%left GREATER GREATER_EQUAL LESSER LESSER_EQUAL EQUAL NOT_EQUAL
%left ADD SUB
%right NOT
%left MUL DIV AND OR
%left POW
%left SQUOTE

%%

program: statements                                                                 { $$ = ProgramGrammarAction($1); }
    ;

statement: evaluate SEMICOLON
    | integrate SEMICOLON
    | derivative SEMICOLON
    | declare_function SEMICOLON
    | if_statement
    | for_statement
    | while_statement
    | print SEMICOLON
    | declare_variable SEMICOLON
    | assign_variable SEMICOLON
    ;

expression: positive_constant
    | evaluate
    | integrate
    | derivative
    | ID
    | expression ADD expression
    | expression SUB expression
    | expression MUL expression
    | expression DIV expression
    | expression POW expression
    | OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    | SUB expression
    ;

domain: expression LESSER expression
    | expression LESSER_EQUAL expression
    | expression GREATER_EQUAL expression
    | expression GREATER expression
    | expression EQUAL expression
    | expression NOT_EQUAL expression
    | expression LESSER expression LESSER expression
    | expression LESSER_EQUAL expression LESSER expression
    | expression LESSER expression LESSER_EQUAL expression
    | expression LESSER_EQUAL expression LESSER_EQUAL expression
    ;

boolean: domain
    | boolean OR boolean
    | boolean AND boolean
    | OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS
    | NOT boolean
    ;

positive_constant: DOUBLE
    | INTEGER
    ;

declare_function: FUNCTION ID ASSIGN function
    | FUNCTION ID ASSIGN OPEN_BRACES functions CLOSE_BRACES
    ;

function: expression
    | expression COLON domain
    | composite
    | taylor
    ;

functions: function
    | function COMMA functions
    ;

composite: ID COMPOSITE ID
    | composite COMPOSITE ID
    ;

expressions: expression
    | expression COMMA expressions
    ;

ids: ID
    | ID COMMA ids
    ;

numArray: OPEN_BRACKETS expressions CLOSE_BRACKETS
    ;

numArrays: numArray
    | numArray COMMA numArrays
    ;

arrayNumArray: OPEN_BRACKETS numArrays CLOSE_BRACKETS
    ;

idArray: OPEN_BRACKETS ids CLOSE_BRACKETS
    ;

escapeString: ESCAPESTRING
    | ESCAPESTRING ADD escapeString
    ;

escapeStrings: escapeString
    | escapeString COMMA escapeStrings
    ;

escapeStringArray: OPEN_BRACKETS escapeStrings CLOSE_BRACKETS
    ;

evaluate: EVALUATE ID IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    | EVALUATE ID IN numArray
    | ID OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    | EVALUATE idArray IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    | EVALUATE idArray IN numArray
    | idArray OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    ;

integrate: INTEGRATE ID BETWEEN OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS
    | INTEGRATE ID OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS
    | INTEGRATE idArray BETWEEN OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS
    | INTEGRATE idArray OPEN_PARENTHESIS expression AND_INT expression CLOSE_PARENTHESIS
    | integrate ERR positive_constant
    ;

squotes: SQUOTE
    | SQUOTE squotes
    ;

print: PRINT OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    | PRINT OPEN_PARENTHESIS escapeString CLOSE_PARENTHESIS
    ;

statements: statement
    | statement statements
    ;

derivative: DERIVATIVE ID IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    | DERIVATIVE INTEGER ID IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    | ID squotes OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    | DERIVATIVE idArray IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    | DERIVATIVE INTEGER idArray IN OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    | idArray squotes OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
    | derivative ERR positive_constant
    ;

block: OPEN_BRACES statements CLOSE_BRACES
    | statement
    ;

if_statement: IF OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS block
    | IF OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS block ELSE block
    ;

for_statement: FOR OPEN_PARENTHESIS declare_variable SEMICOLON boolean SEMICOLON assign_variable CLOSE_PARENTHESIS block
    | FOR OPEN_PARENTHESIS SEMICOLON boolean SEMICOLON assign_variable CLOSE_PARENTHESIS block
    ;

taylor: TAYLOR INTEGER ID
    ;

while_statement: WHILE OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS block
    ;

declare_variable: VAR ID
    | declare_variable ASSIGN expression
    ;

assign_variable: ID ADD_ASSIGN expression
    | ID SUB_ASSIGN expression
    | ID MUL_ASSIGN expression
    | ID DIV_ASSIGN expression
    | ID INCREMENT
    | ID DECREMENT
    | ID ASSIGN expression
    ;

%%
